function [n, dim, setup_mode, setup_params] = ...
    cshelper_parse_grouping_and_dim( cs, data, state, varargin )

% [n, dim, setup_mode, setup_params] = cshelper_parse_grouping_and_dim(
%     cs, data, state, [ 'var_percentage', n |
%                        'initial_grouping', g, ['allow_partial_grouping'] |
%                        'initial_groupings', g ] )
%
% This helper function resolves the requested initial grouping setup and the
% data dimensionality to be used. 'cs' must already contain the modeler
% name. Available arguments are explained in the help section of ivga(). Note
% that if a state is given, then its grouping will be used and these parameters
% are ignored.

% Note: n seems to be still set according to the provided data, even if a state
% struct was given..??


%  This file is a part of the Independent Variable Group Analysis package
% 
%  Copyright (C) 2001-2006 Esa Alhoniemi, Antti Honkela, Krista Lagus,
%  Jeremias Seppa, Harri Valpola, and Paul Wagner
% 
%  This program is free software; you can redistribute it and/or modify
%  it under the terms of the GNU General Public License as published by
%  the Free Software Foundation; either version 2, or (at your option)
%  any later version.
% 
%  This program is distributed in the hope that it will be useful,
%  but WITHOUT ANY WARRANTY; without even the implied warranty of
%  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%  GNU General Public License (included in file License.txt in the
%  program package) for more details.


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


% compute the datapoint count and dimension
[n,dim] = get_data_dimension( cs.mdl.class, data );
if ~isempty(state) && ~isempty(state.vars), dim = length(state.vars); end

% read args
[flg_var_percentage, var_percentage] = isarg('var_percentage');
[flg_grouping, grouping]             = isarg('initial_grouping');
[flg_set_of_groupings, groupings]    = isarg('initial_groupings');
flg_allow_partial_grouping           = isarg('allow_partial_grouping');


% check the group setup mode and correct the dimension if needed

if flg_var_percentage
  
  % percentage mode
  setup_mode = 'percentage'; setup_params = var_percentage;
  dim = max( 2, (var_percentage/100) * dim );
  
elseif flg_grouping
  
  % explicit grouping is given
  setup_mode = 'explicit'; setup_params = grouping;
  grouping_vars = sort([ setup_params{:} ]);   % for the checks below
  
  % check that the grouping does not contain duplicate or non-existing vars
  if length( grouping_vars ) ~= length( unique( grouping_vars ) )
    error([ 'The grouping provided with the ''initial_grouping'' option ' ...
            'contains duplicate variables!' ]);
  end
  if ~all( ismember( grouping_vars, 1:dim ) )
    error([ 'The grouping provided with the ''initial_grouping'' option ' ...
            'contains variables that are not in the data set!' ]);
  end
  
  % check that all variables are included, unless the
  % 'allow_partial_grouping' flag is set
  if ~flg_allow_partial_grouping && ...
        ~isequal( grouping_vars, 1:dim )
    error([ 'The grouping provided with the ''initial_grouping'' option ' ...
            'does not contain all variables of the data set!' ]);
  end
  
  % update the dimensionality
  dim = length( [setup_params{:}] );
  
elseif flg_set_of_groupings
  
  % a set of groupings is given, synchronize with calling batch_run
  
  % extract data from batch_run
  [flg_batchdata, batchdata] = isarg('batch_run_data');
  if ~flg_batchdata
    error('The mode ''initial_groupings'' can be used only with batch_run!');
  end
  
  % set mode to 'explicit'
  setup_mode = 'explicit';
  
  % syntax a? convert to syntax b
  if iscell( groupings ) && ischar( groupings{1} ) && ...
        strcmp( groupings{1}, 'source' )
    
    % use entry 1 if none was specified
    if length(groupings) == 2, groupings{3} = 1; end
    
    % parse args
    source = groupings{2};
    entry  = groupings{3};
    
    % retrieve all groupings from the source
    [groupings, names, std_dims] = ...
        collect_from_results( source, 'grouping' );
    
    % select the requested entry
    if size( groupings, 1 ) < entry
      error([ 'Result entry ' num2str(entry) ' was requested, but the ' ...
              'provided results do not contain that much entries!' ]);
    end
    groupings = shiftdim( groupings(entry,:,:), 1 );
    
  end
  
  % check syntax and generate the grouping
  if iscell( groupings ) && ndims( groupings ) == 2 && ...
        iscell( groupings{1,1} ) && isnumeric( groupings{1,1}{1} )
    % syntax b, synchronize with calling batch_run
    setup_params = groupings{batchdata.dataset_ind, batchdata.iter};
    dim = length( [setup_params{:}] );
  else
    % bad syntax
    error( 'Bad grouping syntax!' );
  end
  
else
  
  % default mode
  setup_mode = 'percentage'; setup_params = 100;
  
end


return;
